#include <stdio.h>
#include <ssslib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdlib.h>

int main(int argc, char *argv[]){

  int sockfd,cli_sock,rc;
  struct sockaddr_in addr;
  struct sockaddr_in cli_addr;
  socklen_t sin_size;
  char *recvbuf;
  char *handle;
  SSS sss;
  int yes=1;
  int i=0;
  char sendbuf[128];
  char errbuf[MAXERRLENGTH+1];
  int type;
  int j;

  if (argc != 4){
    fprintf(stderr,"Usage: server_test <conffile> <port> <protocol>\n");
    exit(1);
  }
  
  addr.sin_family = AF_INET;
  addr.sin_port = htons(atoi(argv[2]));
  addr.sin_addr.s_addr = INADDR_ANY;
  memset(&(addr.sin_zero), '\0', 8);  
  
  sss = SSS_Init(argv[1],1);

  if (!sss){
    printf("Error in SSS init!\n");
    return 1;
  }

  rc = SSS_SDRegister(sss,"test_service","localhost",atoi(argv[2]),argv[3],NULL,NULL);

  if (rc == -1){
    SSS_strerror(sss->sss_errno,errbuf,&type);
    printf("error: %s\n",errbuf);
    return 1;
  }
  
  printf("rc from sdregister was %i\n",rc);
    
  sockfd = socket(AF_INET,SOCK_STREAM,0);

  setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int));

  rc = bind(sockfd,(struct sockaddr *)&addr,sizeof(struct sockaddr));
  
  printf("rc from bind is %i\n",rc);

  listen(sockfd,5);

  sin_size = sizeof(struct sockaddr_in);

  for(j=0;j<5;j++){

    cli_sock = accept(sockfd,(struct sockaddr *)&cli_addr,&sin_size);

    handle = SSS_ServerInit(sss,cli_sock);

    if (!handle){
      SSS_strerror(sss->sss_errno,errbuf,&type);
      printf("error: %s\n",errbuf);
      return 1;
    }

    while ((recvbuf = SSS_RecvMessage(sss,handle))){

      printf("got :%s: from client\n",recvbuf);

      free(recvbuf);

      sprintf(sendbuf,"Test server send %i\n",i);

      rc = SSS_SendMessage(sss,handle,sendbuf);

      if (rc == -1){
	SSS_strerror(sss->sss_errno,errbuf,&type);
	printf("error: %s\n",errbuf);
	return 1;
      }
  
      i++;

    }

    if (!recvbuf){
      if (sss->sss_errno != SSS_EOF){
	SSS_strerror(sss->sss_errno,errbuf,&type);
	printf("error: %s\n",errbuf);
      }
    }
  
    rc = SSS_ServerClose(sss,handle);

    if (rc == -1){
      SSS_strerror(sss->sss_errno,errbuf,&type);
      printf("error: %s\n",errbuf);
      return 1;
    }
  
  }
    
  SSS_Close(sss);

  return 0;
}

